#Filtering BLAST results
#See readme for more detailed instructions

#-------------------------------------
#Words to be excluded

#Type: List of strings
#Description: Results with these words in their species name will be excluded from the final list
#I only wanted results with a defined species (ie. Yersinia intermedia), so I excluded any results
#where the species was not determined (ie. uncultured bacteria)
#Add or delete words to the list according to your preferences

forbidden = ["symbiont", "[", "]", "uncultured", "sp.", "unclassified", "proteobacterium", "/", "bacterium"]

#-------------------------------------
#Makes list of species (Works specifically with the NCBI and Uniprot format)

#Input: Integer > 0 -Minimum size of protein to be accepted
#Output: List of lists
#Description: Opens the file that needs to be filtered, and creates a list for every species that
#includes the NCBI/Uniprot identifier, the name of the species, and its sequence. Keeps track of which species it has seen and
#only adds new ones. Excludes species that include words of the forbidden list

def makeSpecies(minLen,database):
   data = open('fileToFilter.txt', 'r') #open file
   wordList = data.readlines() #set file mode to 'read'

   spList = [] #Final filtered list that will hold a list for every species
   newSp = "" #Name of species
   newId = "" #NCBI identifier
   seq = [] #List to hold every line of the species' sequence
   
   foundSps = [] #List to keep track of the species names the code has seen
   repeats = []

   #For-loop reads every line in the file and organizes it into a list for every species
   for w in range (0,len(wordList)): #w is the index
      
       if (wordList[w])[0] == ">":
          tempLine = wordList[w].split()#Variable to temporarily hold the current line for manipulation
          seq = [] #Re-initialize the seq list

          if database == "N": #Using NCBI BLAST
             newId = tempLine[0] #Make the current word the NCBI id
             for i in range(0, len(tempLine)): #Loop through every word in the line to find the species name
                if (tempLine[i])[0] == "[" and (tempLine[i])[len(tempLine[i])-1] == "]":
                   newSp = tempLine[i] #Make species name
                   newSp = newSp.replace("[","") #Gets rid of first bracket
                   newSp = newSp.replace("]","") #Gets rid of second bracket
                elif (tempLine[i])[0] == "[":
                   newSp = tempLine[i] + " " + tempLine[i+1] #Make species name
                   newSp = newSp.replace("[","") #Gets rid of first bracket
                   newSp = newSp.replace("]","") #Gets rid of second bracket
                   
          elif database == "U": #Using Uniprot BLAST
             lineList = tempLine[0].replace("|"," ") #Separate identifier
             lineList = lineList.split()
             newId = ">" + lineList[1] #Make the current word the Uniprot id
             for i in range(0, len(tempLine)): #Loop through every word in the line to find the species name
                if (tempLine[i])[:3] == "OS=":
                    newSp = (tempLine[i])[3:] + " " + tempLine[i+1] #Make species name 
              
       else: #In the sequence
          seqLine = wordList[w].replace("\n","")#Gets rid of line break
          seq.append(seqLine)#Add line to sequence list
          if w == len(wordList)-1 or (wordList[w+1])[0] == ">": #next line is the start of another species or reached end of file
             if newSp not in foundSps: #Confirms that the species name hasn't been seen before
                spList.append([newId,newSp,seq]) #Add entry to final list
                foundSps.append(newSp) #Add species name to the list of species seen
                
   newFile = open("filtered.txt","w") #Make a new text file 

   for entry in spList:
      checkSp = entry[1].split() #Turn the species name into a list of words (makes it easier to compare with the forbidden list)
      checkSeq = "".join(entry[2]) #Join the list of lines that make up the sequence into a single line (makes it easier to check size)
      
      #Only add to file if the species name is not forbidden,the species name has at least two words, and the sequence has the required minimum length
      if len(checkSp) > 1 and all(item not in checkSp for item in forbidden) and len(checkSeq) >= minLen:
         newFile.write(entry[0] + " " + entry[1] + "\n") #In the first line add the NCBI id and species name
         for line in entry[2]: #Add the sequence
            newFile.write(line+ "\n")
   newFile.close()
   data.close()
   
#-------------------------------------


#User interface
      
marker = 1
while marker == 1:
    print ("*****************************************************************************")
    print ("*****************************************************************************")
    print ("*****************************************************************************")
    print ("*****************************************************************************")
    print ("Welcome user!")
    print (     "Press 1 to run Filter")
    print (     "Press 0 to Quit")

    
    selection = raw_input("Run Filter?")
      
    if selection == "1":
       
        minLenTemp = raw_input("What is the minimum length accepted? ")
        correctMinLen = 0
        while correctMinLen == 0:
           if minLenTemp.isdigit() and int(minLenTemp) >= 1:
              minLen = int(minLenTemp)
              correctMinLen = 1
           else:
              print ("")
              print ("")
              print ("")
              print ("")
              print ("***********Only integers greater than 0 please***********")
              print ("")
              print ("")
              print ("")
              print ("")
              minLenTemp = raw_input("What is the minimum length accepted? ")

        databaseTemp = raw_input("Which webpage did you use? ")
        correctDatabase = 0
        while correctDatabase == 0:
           if databaseTemp == "N" or databaseTemp == "U":
              database = databaseTemp
              correctDatabase = 1
           else:
              print ("")
              print ("")
              print ("")
              print ("")
              print ("***********Only N or U please***********")
              print ("")
              print ("")
              print ("")
              print ("")
              databaseTemp = raw_input("Which webpage did you use? ")
   

        makeSpecies(minLen,database)
        
    elif selection == "0":
        marker = 0
        
    else:
        print ("")
        print ("")
        print ("")
        print ("")
        print ("***********Only 1 or 0 please***********")
        print ("")
        print ("")
        print ("")
        print ("")
